# Import and register the core plugins

from rekall.plugins import addrspaces
from rekall.plugins import core
from rekall.plugins import darwin
from rekall.plugins import guess_profile
from rekall.plugins import imagecopy
from rekall.plugins import linux
from rekall.plugins import overlays
from rekall.plugins import tools
from rekall.plugins import windows
