
# AUTOGENERATED File do not edit!
# File generated by the profile_tool.py 'compress' command.

from rekall import io_manager


class BuiltInProfiles(io_manager.BuiltInManager):
    data = {
 'Win7SP1x64': {
  'Constants.json': {
   'DRIVER_POOLTAG': 'Dri\xf6',
   'EPROCESS_POOLTAG': 'Pro\xe3',
   'FILE_POOLTAG': 'Fil\xe5',
   'MODULE_POOLTAG': 'MmLd',
   'MUTANT_POOLTAG': 'Mut\xe1',
   'MaxPointer': 281474976710655,
   'PoolAlignment': 16,
   'SYMLINK_POOLTAG': 'Sym\xe2',
   'THREAD_POOLTAG': 'Thr\xe5',
   'default_text_encoding': 'utf-16-le',
  },
  'metadata': {
   'Constants': 'Constants.json',
   'ProfileClass': 'Win7SP1x64',
   'VTypes': 'vtypes.json',
  },
  'vtypes.json': {
   'LIST_ENTRY64': [16, {
    'Blink': [8, ['unsigned long long']],
    'Flink': [0, ['unsigned long long']],
   }],
   '_ALPC_PROCESS_CONTEXT': [32, {
   }],
   '_CHILD_LIST': [8, {
    'Count': [0, ['unsigned long']],
    'List': [4, ['unsigned long']],
   }],
   '_CLIENT_ID': [16, {
    'UniqueProcess': [0, ['pointer64', ['void']]],
    'UniqueThread': [8, ['pointer64', ['void']]],
   }],
   '_CMHIVE': [3048, {
    'FileFullPath': [1752, ['_UNICODE_STRING']],
    'FileUserName': [1768, ['_UNICODE_STRING']],
    'Hive': [0, ['_HHIVE']],
    'HiveList': [1496, ['_LIST_ENTRY']],
    'HiveRootPath': [1784, ['_UNICODE_STRING']],
   }],
   '_CM_KEY_INDEX': [8, {
    'Count': [2, ['unsigned short']],
    'List': [4, ['array', 1, ['unsigned long']]],
    'Signature': [0, ['unsigned short']],
   }],
   '_CM_KEY_NODE': [80, {
    'Flags': [2, ['unsigned short']],
    'LastWriteTime': [4, ['_LARGE_INTEGER']],
    'Name': [76, ['array', 1, ['wchar']]],
    'NameLength': [72, ['unsigned short']],
    'Signature': [0, ['unsigned short']],
    'SubKeyCounts': [20, ['array', 2, ['unsigned long']]],
    'SubKeyLists': [28, ['array', 2, ['unsigned long']]],
    'ValueList': [36, ['_CHILD_LIST']],
   }],
   '_CM_KEY_VALUE': [24, {
    'Data': [8, ['unsigned long']],
    'DataLength': [4, ['unsigned long']],
    'Name': [20, ['array', 1, ['wchar']]],
    'NameLength': [2, ['unsigned short']],
    'Signature': [0, ['unsigned short']],
    'Type': [12, ['unsigned long']],
   }],
   '_CONTROL_AREA': [128, {
    'DereferenceList': [8, ['_LIST_ENTRY']],
    'FilePointer': [64, ['_EX_FAST_REF']],
    'NumberOfMappedViews': [40, ['unsigned long long']],
    'NumberOfPfnReferences': [32, ['unsigned long long']],
    'NumberOfSectionReferences': [24, ['unsigned long long']],
    'NumberOfUserReferences': [48, ['unsigned long long']],
    'Segment': [0, ['pointer64', ['_SEGMENT']]],
    'WaitingForDeletion': [80, ['pointer64', ['_MI_SECTION_CREATION_GATE']]],
    'u': [56, ['__unnamed_157b']],
   }],
   '_DBGKD_DEBUG_DATA_HEADER64': [24, {
    'List': [0, ['LIST_ENTRY64']],
    'OwnerTag': [16, ['String', {
     'length': 4,
    }]],
   }],
   '_DEVICE_OBJECT': [336, {
    'AttachedDevice': [24, ['pointer64', ['_DEVICE_OBJECT']]],
    'DeviceType': [72, ['unsigned long']],
    'DriverObject': [8, ['pointer64', ['_DRIVER_OBJECT']]],
    'NextDevice': [16, ['pointer64', ['_DEVICE_OBJECT']]],
   }],
   '_DISPATCHER_HEADER': [24, {
    'SignalState': [4, ['long']],
    'Type': [0, ['unsigned char']],
   }],
   '_DRIVER_EXTENSION': [56, {
    'ServiceKeyName': [24, ['_UNICODE_STRING']],
   }],
   '_DRIVER_OBJECT': [336, {
    'DeviceObject': [8, ['pointer64', ['_DEVICE_OBJECT']]],
    'DriverName': [56, ['_UNICODE_STRING']],
    'DriverSize': [32, ['unsigned long']],
    'DriverStart': [24, ['pointer64', ['void']]],
    'DriverStartIo': [96, ['pointer64', ['void']]],
    'MajorFunction': [112, ['array', 28, ['pointer64', ['void']]]],
   }],
   '_DUAL': [632, {
    'Length': [0, ['unsigned long']],
    'Map': [8, ['pointer64', ['_HMAP_DIRECTORY']]],
   }],
   '_EPROCESS': [1232, {
    'AccountingFolded': [1084, ['BitField', {
     'end_bit': 2,
     'native_type': 'unsigned long',
     'start_bit': 1,
    }]],
    'ActiveProcessLinks': [392, ['_LIST_ENTRY']],
    'ActiveThreads': [808, ['unsigned long']],
    'ActiveThreadsHighWatermark': [1212, ['unsigned long']],
    'AddressCreationLock': [536, ['_EX_PUSH_LOCK']],
    'AddressSpaceInitialized': [1088, ['BitField', {
     'end_bit': 12,
     'native_type': 'unsigned long',
     'start_bit': 10,
    }]],
    'AffinityPermanent': [1084, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 18,
    }]],
    'AffinityUpdateEnable': [1084, ['BitField', {
     'end_bit': 20,
     'native_type': 'unsigned long',
     'start_bit': 19,
    }]],
    'AlpcContext': [1160, ['_ALPC_PROCESS_CONTEXT']],
    'AweInfo': [904, ['pointer64', ['void']]],
    'BreakOnTermination': [1088, ['BitField', {
     'end_bit': 14,
     'native_type': 'unsigned long',
     'start_bit': 13,
    }]],
    'CloneRoot': [576, ['pointer64', ['void']]],
    'CommitCharge': [440, ['unsigned long long']],
    'CommitChargeLimit': [888, ['unsigned long long']],
    'CommitChargePeak': [896, ['unsigned long long']],
    'ConsoleHostProcess': [680, ['unsigned long long']],
    'Cookie': [632, ['unsigned long']],
    'CpuQuotaBlock': [456, ['pointer64', ['_PS_CPU_QUOTA_BLOCK']]],
    'CreateReported': [1088, ['BitField', {
     'end_bit': 1,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'CreateTime': [360, ['_LARGE_INTEGER']],
    'CrossSessionCreate': [1088, ['BitField', {
     'end_bit': 26,
     'native_type': 'unsigned long',
     'start_bit': 25,
    }]],
    'DebugPort': [496, ['pointer64', ['void']]],
    'DefaultHardErrorProcessing': [816, ['unsigned long']],
    'DefaultIoPriority': [1088, ['BitField', {
     'end_bit': 30,
     'native_type': 'unsigned long',
     'start_bit': 27,
    }]],
    'DefaultPagePriority': [1084, ['BitField', {
     'end_bit': 15,
     'native_type': 'unsigned long',
     'start_bit': 12,
    }]],
    'DeprioritizeViews': [1088, ['BitField', {
     'end_bit': 15,
     'native_type': 'unsigned long',
     'start_bit': 14,
    }]],
    'DeviceMap': [688, ['pointer64', ['void']]],
    'EtwDataSource': [696, ['pointer64', ['void']]],
    'ExceptionPortData': [504, ['pointer64', ['void']]],
    'ExceptionPortState': [504, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned long long',
     'start_bit': 0,
    }]],
    'ExceptionPortValue': [504, ['unsigned long long']],
    'ExitProcessReported': [1084, ['BitField', {
     'end_bit': 4,
     'native_type': 'unsigned long',
     'start_bit': 3,
    }]],
    'ExitStatus': [1092, ['long']],
    'ExitTime': [368, ['_LARGE_INTEGER']],
    'ExplicitAffinity': [1084, ['BitField', {
     'end_bit': 22,
     'native_type': 'unsigned long',
     'start_bit': 21,
    }]],
    'Filler': [720, ['unsigned long long']],
    'Flags': [1088, ['unsigned long']],
    'Flags2': [1084, ['unsigned long']],
    'ForkFailed': [1088, ['BitField', {
     'end_bit': 9,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
    'ForkInProgress': [552, ['pointer64', ['_ETHREAD']]],
    'FreeTebHint': [704, ['pointer64', ['void']]],
    'FreeUmsTebHint': [712, ['pointer64', ['void']]],
    'HandleTableRundown': [1084, ['BitField', {
     'end_bit': 8,
     'native_type': 'unsigned long',
     'start_bit': 7,
    }]],
    'HardwareTrigger': [560, ['unsigned long long']],
    'HasAddressSpace': [1088, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 18,
    }]],
    'HighestUserAddress': [1072, ['pointer64', ['void']]],
    'ImageFileName': [736, ['array', 15, ['unsigned char']]],
    'ImageNotifyDone': [1088, ['BitField', {
     'end_bit': 23,
     'native_type': 'unsigned long',
     'start_bit': 22,
    }]],
    'ImagePathHash': [812, ['unsigned long']],
    'InheritedFromUniqueProcessId': [656, ['pointer64', ['void']]],
    'InjectInpageErrors': [1088, ['BitField', {
     'end_bit': 21,
     'native_type': 'unsigned long',
     'start_bit': 20,
    }]],
    'Job': [608, ['pointer64', ['_EJOB']]],
    'JobLinks': [752, ['_LIST_ENTRY']],
    'JobNotReallyActive': [1084, ['BitField', {
     'end_bit': 1,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'LastReportMemory': [1084, ['BitField', {
     'end_bit': 6,
     'native_type': 'unsigned long',
     'start_bit': 5,
    }]],
    'LastThreadExitStatus': [820, ['long']],
    'LaunchPrefetched': [1088, ['BitField', {
     'end_bit': 20,
     'native_type': 'unsigned long',
     'start_bit': 19,
    }]],
    'LdtInformation': [664, ['pointer64', ['void']]],
    'LockedPagesList': [768, ['pointer64', ['void']]],
    'MmProcessLinks': [1056, ['_LIST_ENTRY']],
    'ModifiedPageCount': [1080, ['unsigned long']],
    'NeedsHandleRundown': [1084, ['BitField', {
     'end_bit': 9,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
    'NewProcessReported': [1084, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned long',
     'start_bit': 2,
    }]],
    'NoDebugInherit': [1088, ['BitField', {
     'end_bit': 2,
     'native_type': 'unsigned long',
     'start_bit': 1,
    }]],
    'NumaAware': [1084, ['BitField', {
     'end_bit': 11,
     'native_type': 'unsigned long',
     'start_bit': 10,
    }]],
    'NumberOfLockedPages': [592, ['unsigned long long']],
    'NumberOfPrivatePages': [584, ['unsigned long long']],
    'ObjectTable': [512, ['pointer64', ['_HANDLE_TABLE']]],
    'OtherOperationCount': [856, ['_LARGE_INTEGER']],
    'OtherTransferCount': [880, ['_LARGE_INTEGER']],
    'OutswapEnabled': [1088, ['BitField', {
     'end_bit': 7,
     'native_type': 'unsigned long',
     'start_bit': 6,
    }]],
    'Outswapped': [1088, ['BitField', {
     'end_bit': 8,
     'native_type': 'unsigned long',
     'start_bit': 7,
    }]],
    'OverrideAddressSpace': [1088, ['BitField', {
     'end_bit': 18,
     'native_type': 'unsigned long',
     'start_bit': 17,
    }]],
    'PageDirectoryPte': [720, ['_HARDWARE_PTE']],
    'Pcb': [0, ['_KPROCESS']],
    'PdeUpdateNeeded': [1088, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 23,
    }]],
    'PeakVirtualSize': [464, ['unsigned long long']],
    'Peb': [824, ['pointer64', ['_PEB']]],
    'PhysicalVadRoot': [568, ['pointer64', ['_MM_AVL_TABLE']]],
    'PrefetchTrace': [832, ['_EX_FAST_REF']],
    'PrimaryTokenFrozen': [1084, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned long',
     'start_bit': 15,
    }]],
    'PriorityClass': [751, ['unsigned char']],
    'ProcessDelete': [1088, ['BitField', {
     'end_bit': 4,
     'native_type': 'unsigned long',
     'start_bit': 3,
    }]],
    'ProcessExiting': [1088, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned long',
     'start_bit': 2,
    }]],
    'ProcessInSession': [1088, ['BitField', {
     'end_bit': 17,
     'native_type': 'unsigned long',
     'start_bit': 16,
    }]],
    'ProcessInserted': [1088, ['BitField', {
     'end_bit': 27,
     'native_type': 'unsigned long',
     'start_bit': 26,
    }]],
    'ProcessLock': [352, ['_EX_PUSH_LOCK']],
    'ProcessQuotaPeak': [424, ['array', 2, ['unsigned long long']]],
    'ProcessQuotaUsage': [408, ['array', 2, ['unsigned long long']]],
    'ProcessSelfDelete': [1088, ['BitField', {
     'end_bit': 31,
     'native_type': 'unsigned long',
     'start_bit': 30,
    }]],
    'ProcessVerifierTarget': [1084, ['BitField', {
     'end_bit': 17,
     'native_type': 'unsigned long',
     'start_bit': 16,
    }]],
    'PropagateNode': [1084, ['BitField', {
     'end_bit': 21,
     'native_type': 'unsigned long',
     'start_bit': 20,
    }]],
    'ProtectedProcess': [1084, ['BitField', {
     'end_bit': 12,
     'native_type': 'unsigned long',
     'start_bit': 11,
    }]],
    'QuotaBlock': [448, ['pointer64', ['_EPROCESS_QUOTA_BLOCK']]],
    'ReadOperationCount': [840, ['_LARGE_INTEGER']],
    'ReadTransferCount': [864, ['_LARGE_INTEGER']],
    'RefTraceEnabled': [1084, ['BitField', {
     'end_bit': 10,
     'native_type': 'unsigned long',
     'start_bit': 9,
    }]],
    'ReportCommitChanges': [1084, ['BitField', {
     'end_bit': 5,
     'native_type': 'unsigned long',
     'start_bit': 4,
    }]],
    'ReportPhysicalPageChanges': [1084, ['BitField', {
     'end_bit': 7,
     'native_type': 'unsigned long',
     'start_bit': 6,
    }]],
    'RequestedTimerResolution': [1208, ['unsigned long']],
    'RotateInProgress': [544, ['pointer64', ['_ETHREAD']]],
    'RundownProtect': [376, ['_EX_RUNDOWN_REF']],
    'SeAuditProcessCreationInfo': [912, ['_SE_AUDIT_PROCESS_CREATION_INFO']],
    'SectionBaseAddress': [624, ['pointer64', ['void']]],
    'SectionObject': [616, ['pointer64', ['void']]],
    'SecurityPort': [792, ['pointer64', ['void']]],
    'Session': [728, ['pointer64', ['void']]],
    'SessionProcessLinks': [480, ['_LIST_ENTRY']],
    'SetTimerResolution': [1088, ['BitField', {
     'end_bit': 13,
     'native_type': 'unsigned long',
     'start_bit': 12,
    }]],
    'SetTimerResolutionLink': [1088, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 31,
    }]],
    'SmallestTimerResolution': [1216, ['unsigned long']],
    'Spare': [672, ['pointer64', ['void']]],
    'StackRandomizationDisabled': [1084, ['BitField', {
     'end_bit': 18,
     'native_type': 'unsigned long',
     'start_bit': 17,
    }]],
    'ThreadListHead': [776, ['_LIST_ENTRY']],
    'TimerResolutionLink': [1192, ['_LIST_ENTRY']],
    'TimerResolutionStackRecord': [1224, ['pointer64', ['_PO_DIAG_STACK_RECORD']]],
    'Token': [520, ['_EX_FAST_REF']],
    'UmsScheduledThreads': [636, ['unsigned long']],
    'UniqueProcessId': [384, ['pointer64', ['void']]],
    'VadRoot': [1096, ['_MM_AVL_TABLE']],
    'VdmAllowed': [1088, ['BitField', {
     'end_bit': 25,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
    'VirtualSize': [472, ['unsigned long long']],
    'Vm': [920, ['_MMSUPPORT']],
    'VmDeleted': [1088, ['BitField', {
     'end_bit': 6,
     'native_type': 'unsigned long',
     'start_bit': 5,
    }]],
    'VmTopDown': [1088, ['BitField', {
     'end_bit': 22,
     'native_type': 'unsigned long',
     'start_bit': 21,
    }]],
    'Win32Process': [600, ['pointer64', ['void']]],
    'Win32WindowStation': [648, ['pointer64', ['void']]],
    'WorkingSetPage': [528, ['unsigned long long']],
    'WorkingSetWatch': [640, ['pointer64', ['_PAGEFAULT_HISTORY']]],
    'Wow64Process': [800, ['pointer64', ['void']]],
    'Wow64SplitPages': [1088, ['BitField', {
     'end_bit': 5,
     'native_type': 'unsigned long',
     'start_bit': 4,
    }]],
    'Wow64VaSpace4Gb': [1088, ['BitField', {
     'end_bit': 10,
     'native_type': 'unsigned long',
     'start_bit': 9,
    }]],
    'WriteOperationCount': [848, ['_LARGE_INTEGER']],
    'WriteTransferCount': [872, ['_LARGE_INTEGER']],
    'WriteWatch': [1088, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned long',
     'start_bit': 15,
    }]],
   }],
   '_ETHREAD': [1176, {
    'Cid': [944, ['_CLIENT_ID']],
    'CreateTime': [864, ['_LARGE_INTEGER']],
    'ExitTime': [872, ['_LARGE_INTEGER']],
    'KeyedWaitSemaphore': [960, ['_KSEMAPHORE']],
    'StartAddress': [904, ['pointer64', ['void']]],
    'Tcb': [0, ['_KTHREAD']],
   }],
   '_EX_FAST_REF': [8, {
    'Object': [0, ['pointer64', ['void']]],
   }],
   '_EX_PUSH_LOCK': [8, {
   }],
   '_EX_RUNDOWN_REF': [8, {
   }],
   '_FILE_OBJECT': [216, {
    'DeleteAccess': [76, ['unsigned char']],
    'FileName': [88, ['_UNICODE_STRING']],
    'ReadAccess': [74, ['unsigned char']],
    'SharedDelete': [79, ['unsigned char']],
    'SharedRead': [77, ['unsigned char']],
    'SharedWrite': [78, ['unsigned char']],
    'WriteAccess': [75, ['unsigned char']],
   }],
   '_HANDLE_TABLE': [104, {
    'HandleCount': [88, ['unsigned long']],
    'TableCode': [0, ['unsigned long long']],
   }],
   '_HANDLE_TABLE_ENTRY': [16, {
    'Object': [0, ['pointer64', ['void']]],
   }],
   '_HARDWARE_PTE': [8, {
   }],
   '_HHIVE': [1432, {
    'BaseBlock': [80, ['pointer64', ['_HBASE_BLOCK']]],
    'Flat': [120, ['unsigned char']],
    'Signature': [0, ['unsigned long']],
    'Storage': [168, ['array', 2, ['_DUAL']]],
   }],
   '_HMAP_DIRECTORY': [8192, {
    'Directory': [0, ['array', 1024, ['pointer64', ['_HMAP_TABLE']]]],
   }],
   '_HMAP_ENTRY': [32, {
    'BlockAddress': [0, ['unsigned long long']],
   }],
   '_HMAP_TABLE': [16384, {
    'Table': [0, ['array', 512, ['_HMAP_ENTRY']]],
   }],
   '_KAPC_STATE': [48, {
    'Process': [32, ['pointer64', ['_KPROCESS']]],
   }],
   '_KDDEBUGGER_DATA64': [832, {
    'Header': [0, ['_DBGKD_DEBUG_DATA_HEADER64']],
    'MmPfnDatabase': [192, ['unsigned long long']],
    'PsActiveProcessHead': [80, ['unsigned long long']],
    'PsLoadedModuleList': [72, ['unsigned long long']],
    'PspCidTable': [88, ['unsigned long long']],
   }],
   '_KMUTANT': [56, {
    'Header': [0, ['_DISPATCHER_HEADER']],
    'OwnerThread': [40, ['pointer64', ['_KTHREAD']]],
   }],
   '_KPCR': [20096, {
    'GdtBase': [0, ['pointer64', ['_KGDTENTRY64']]],
    'IdtBase': [56, ['pointer64', ['_KIDTENTRY64']]],
    'KdVersionBlock': [264, ['pointer64', ['void']]],
    'Prcb': [384, ['_KPRCB']],
    'Self': [24, ['pointer64', ['_KPCR']]],
   }],
   '_KPRCB': [19712, {
    'CurrentThread': [8, ['pointer64', ['_KTHREAD']]],
    'IdleThread': [24, ['pointer64', ['_KTHREAD']]],
    'MHz': [1524, ['unsigned long']],
    'NextThread': [16, ['pointer64', ['_KTHREAD']]],
    'Number': [36, ['unsigned long']],
    'ProcessorState': [64, ['_KPROCESSOR_STATE']],
    'VendorString': [19384, ['array', 13, ['unsigned char']]],
    'WaitListHead': [17536, ['_LIST_ENTRY']],
   }],
   '_KPROCESS': [352, {
    'DirectoryTableBase': [40, ['unsigned long long']],
    'ThreadListHead': [48, ['_LIST_ENTRY']],
   }],
   '_KPROCESSOR_STATE': [1456, {
    'SpecialRegisters': [0, ['_KSPECIAL_REGISTERS']],
   }],
   '_KSEMAPHORE': [32, {
    'Header': [0, ['_DISPATCHER_HEADER']],
   }],
   '_KSPECIAL_REGISTERS': [216, {
    'Cr3': [16, ['unsigned long long']],
   }],
   '_KTHREAD': [864, {
    'ApcState': [80, ['_KAPC_STATE']],
    'Process': [528, ['pointer64', ['_KPROCESS']]],
    'SuspendSemaphore': [728, ['_KSEMAPHORE']],
    'ThreadListEntry': [760, ['_LIST_ENTRY']],
    'WaitListEntry': [160, ['_LIST_ENTRY']],
   }],
   '_LARGE_INTEGER': [8, {
   }],
   '_LDR_DATA_TABLE_ENTRY': [224, {
    'BaseDllName': [88, ['_UNICODE_STRING']],
    'DllBase': [48, ['pointer64', ['void']]],
    'FullDllName': [72, ['_UNICODE_STRING']],
    'InInitializationOrderLinks': [32, ['_LIST_ENTRY']],
    'InLoadOrderLinks': [0, ['_LIST_ENTRY']],
    'InMemoryOrderLinks': [16, ['_LIST_ENTRY']],
    'SizeOfImage': [64, ['unsigned long']],
   }],
   '_LIST_ENTRY': [16, {
    'Blink': [8, ['pointer64', ['_LIST_ENTRY']]],
    'Flink': [0, ['pointer64', ['_LIST_ENTRY']]],
   }],
   '_MMADDRESS_NODE': [40, {
    'LeftChild': [8, ['pointer64', ['_MMADDRESS_NODE']]],
    'RightChild': [16, ['pointer64', ['_MMADDRESS_NODE']]],
   }],
   '_MMPFN': [48, {
    'PteAddress': [16, ['pointer64', ['_MMPTE']]],
    'u1': [0, ['__unnamed_152f']],
    'u3': [24, ['__unnamed_1536']],
    'u4': [40, ['__unnamed_153e']],
   }],
   '_MMPFNENTRY': [2, {
    'PageLocation': [0, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned char',
     'start_bit': 0,
    }]],
   }],
   '_MMSECTION_FLAGS': [4, {
    'Accessed': [0, ['BitField', {
     'end_bit': 15,
     'native_type': 'unsigned long',
     'start_bit': 14,
    }]],
    'Based': [0, ['BitField', {
     'end_bit': 7,
     'native_type': 'unsigned long',
     'start_bit': 6,
    }]],
    'BeingCreated': [0, ['BitField', {
     'end_bit': 2,
     'native_type': 'unsigned long',
     'start_bit': 1,
    }]],
    'BeingDeleted': [0, ['BitField', {
     'end_bit': 1,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'BeingPurged': [0, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned long',
     'start_bit': 2,
    }]],
    'CollidedFlush': [0, ['BitField', {
     'end_bit': 23,
     'native_type': 'unsigned long',
     'start_bit': 22,
    }]],
    'Commit': [0, ['BitField', {
     'end_bit': 14,
     'native_type': 'unsigned long',
     'start_bit': 13,
    }]],
    'CopyOnWrite': [0, ['BitField', {
     'end_bit': 12,
     'native_type': 'unsigned long',
     'start_bit': 11,
    }]],
    'DeleteOnClose': [0, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 18,
    }]],
    'FailAllIo': [0, ['BitField', {
     'end_bit': 5,
     'native_type': 'unsigned long',
     'start_bit': 4,
    }]],
    'File': [0, ['BitField', {
     'end_bit': 8,
     'native_type': 'unsigned long',
     'start_bit': 7,
    }]],
    'FilePointerNull': [0, ['BitField', {
     'end_bit': 20,
     'native_type': 'unsigned long',
     'start_bit': 19,
    }]],
    'GlobalMemory': [0, ['BitField', {
     'end_bit': 18,
     'native_type': 'unsigned long',
     'start_bit': 17,
    }]],
    'GlobalOnlyPerSession': [0, ['BitField', {
     'end_bit': 21,
     'native_type': 'unsigned long',
     'start_bit': 20,
    }]],
    'Image': [0, ['BitField', {
     'end_bit': 6,
     'native_type': 'unsigned long',
     'start_bit': 5,
    }]],
    'Networked': [0, ['BitField', {
     'end_bit': 9,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
    'NoChange': [0, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 23,
    }]],
    'NoModifiedWriting': [0, ['BitField', {
     'end_bit': 4,
     'native_type': 'unsigned long',
     'start_bit': 3,
    }]],
    'PhysicalMemory': [0, ['BitField', {
     'end_bit': 11,
     'native_type': 'unsigned long',
     'start_bit': 10,
    }]],
    'PreferredNode': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 26,
    }]],
    'Reserve': [0, ['BitField', {
     'end_bit': 13,
     'native_type': 'unsigned long',
     'start_bit': 12,
    }]],
    'Rom': [0, ['BitField', {
     'end_bit': 10,
     'native_type': 'unsigned long',
     'start_bit': 9,
    }]],
    'SetMappedFileIoComplete': [0, ['BitField', {
     'end_bit': 22,
     'native_type': 'unsigned long',
     'start_bit': 21,
    }]],
    'Spare': [0, ['BitField', {
     'end_bit': 25,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
    'UserReference': [0, ['BitField', {
     'end_bit': 17,
     'native_type': 'unsigned long',
     'start_bit': 16,
    }]],
    'UserWritable': [0, ['BitField', {
     'end_bit': 26,
     'native_type': 'unsigned long',
     'start_bit': 25,
    }]],
    'WasPurged': [0, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned long',
     'start_bit': 15,
    }]],
   }],
   '_MMSUPPORT': [136, {
   }],
   '_MMVAD': [120, {
    'EndingVpn': [32, ['unsigned long long']],
    'FirstPrototypePte': [80, ['pointer64', ['_MMPTE']]],
    'LastContiguousPte': [88, ['pointer64', ['_MMPTE']]],
    'LeftChild': [8, ['pointer64', ['_MMVAD']]],
    'RightChild': [16, ['pointer64', ['_MMVAD']]],
    'StartingVpn': [24, ['unsigned long long']],
    'Subsection': [72, ['pointer64', ['_SUBSECTION']]],
    'u': [40, ['__unnamed_15c2']],
    'u2': [64, ['__unnamed_15d2']],
   }],
   '_MMVAD_FLAGS': [8, {
    'CommitCharge': [0, ['BitField', {
     'end_bit': 51,
     'native_type': 'unsigned long long',
     'start_bit': 0,
    }]],
    'MemCommit': [0, ['BitField', {
     'end_bit': 56,
     'native_type': 'unsigned long long',
     'start_bit': 55,
    }]],
    'NoChange': [0, ['BitField', {
     'end_bit': 52,
     'native_type': 'unsigned long long',
     'start_bit': 51,
    }]],
    'PrivateMemory': [0, ['BitField', {
     'end_bit': 64,
     'native_type': 'unsigned long long',
     'start_bit': 63,
    }]],
    'Protection': [0, ['BitField', {
     'end_bit': 61,
     'native_type': 'unsigned long long',
     'start_bit': 56,
    }]],
    'Spare': [0, ['BitField', {
     'end_bit': 63,
     'native_type': 'unsigned long long',
     'start_bit': 61,
    }]],
    'VadType': [0, ['BitField', {
     'end_bit': 55,
     'native_type': 'unsigned long long',
     'start_bit': 52,
    }]],
   }],
   '_MMVAD_FLAGS2': [4, {
    'CopyOnWrite': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 31,
    }]],
    'ExtendableFile': [0, ['BitField', {
     'end_bit': 30,
     'native_type': 'unsigned long',
     'start_bit': 29,
    }]],
    'FileOffset': [0, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'Inherit': [0, ['BitField', {
     'end_bit': 31,
     'native_type': 'unsigned long',
     'start_bit': 30,
    }]],
    'LongVad': [0, ['BitField', {
     'end_bit': 29,
     'native_type': 'unsigned long',
     'start_bit': 28,
    }]],
    'MultipleSecured': [0, ['BitField', {
     'end_bit': 27,
     'native_type': 'unsigned long',
     'start_bit': 26,
    }]],
    'OneSecured': [0, ['BitField', {
     'end_bit': 26,
     'native_type': 'unsigned long',
     'start_bit': 25,
    }]],
    'SecNoChange': [0, ['BitField', {
     'end_bit': 25,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
    'Spare': [0, ['BitField', {
     'end_bit': 28,
     'native_type': 'unsigned long',
     'start_bit': 27,
    }]],
   }],
   '_MMVAD_LONG': [144, {
    'EndingVpn': [32, ['unsigned long long']],
    'FirstPrototypePte': [80, ['pointer64', ['_MMPTE']]],
    'LastContiguousPte': [88, ['pointer64', ['_MMPTE']]],
    'LeftChild': [8, ['pointer64', ['_MMVAD']]],
    'RightChild': [16, ['pointer64', ['_MMVAD']]],
    'StartingVpn': [24, ['unsigned long long']],
    'Subsection': [72, ['pointer64', ['_SUBSECTION']]],
    'u': [40, ['__unnamed_15c2']],
    'u2': [64, ['__unnamed_15d2']],
   }],
   '_MMVAD_SHORT': [64, {
    'EndingVpn': [32, ['unsigned long long']],
    'LeftChild': [8, ['pointer64', ['_MMVAD']]],
    'RightChild': [16, ['pointer64', ['_MMVAD']]],
    'StartingVpn': [24, ['unsigned long long']],
    'u': [40, ['__unnamed_15c2']],
   }],
   '_MM_AVL_TABLE': [64, {
    'BalancedRoot': [0, ['_MMADDRESS_NODE']],
   }],
   '_MM_SESSION_SPACE': [8064, {
    'SessionId': [8, ['unsigned long']],
   }],
   '_OBJECT_HANDLE_COUNT_ENTRY': [16, {
    'Process': [0, ['pointer64', ['_EPROCESS']]],
   }],
   '_OBJECT_HEADER': [56, {
    'Body': [48, ['_QUAD']],
    'HandleCount': [8, ['long long']],
    'InfoMask': [26, ['unsigned char']],
    'PointerCount': [0, ['long long']],
    'TypeIndex': [24, ['unsigned char']],
   }],
   '_OBJECT_HEADER_CREATOR_INFO': [32, {
   }],
   '_OBJECT_HEADER_HANDLE_INFO': [16, {
    'SingleEntry': [0, ['_OBJECT_HANDLE_COUNT_ENTRY']],
   }],
   '_OBJECT_HEADER_NAME_INFO': [32, {
    'Name': [8, ['_UNICODE_STRING']],
   }],
   '_OBJECT_HEADER_PROCESS_INFO': [16, {
   }],
   '_OBJECT_HEADER_QUOTA_INFO': [32, {
   }],
   '_OBJECT_SYMBOLIC_LINK': [32, {
    'CreationTime': [0, ['_LARGE_INTEGER']],
    'LinkTarget': [8, ['_UNICODE_STRING']],
   }],
   '_PEB': [896, {
    'CSDVersion': [744, ['_UNICODE_STRING']],
    'ImageBaseAddress': [16, ['pointer64', ['void']]],
    'Ldr': [24, ['pointer64', ['_PEB_LDR_DATA']]],
    'ProcessParameters': [32, ['pointer64', ['_RTL_USER_PROCESS_PARAMETERS']]],
   }],
   '_PEB_LDR_DATA': [88, {
    'InInitializationOrderModuleList': [48, ['_LIST_ENTRY']],
    'InLoadOrderModuleList': [16, ['_LIST_ENTRY']],
    'InMemoryOrderModuleList': [32, ['_LIST_ENTRY']],
   }],
   '_POOL_HEADER': [16, {
    'BlockSize': [0, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 16,
    }]],
    'PoolIndex': [0, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
    'PoolTag': [4, ['unsigned long']],
    'PoolType': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
   }],
   '_QUAD': [8, {
   }],
   '_RTL_USER_PROCESS_PARAMETERS': [1024, {
    'CommandLine': [112, ['_UNICODE_STRING']],
    'Environment': [128, ['pointer64', ['void']]],
   }],
   '_SE_AUDIT_PROCESS_CREATION_INFO': [8, {
   }],
   '_SID': [12, {
    'IdentifierAuthority': [2, ['_SID_IDENTIFIER_AUTHORITY']],
    'Revision': [0, ['unsigned char']],
    'SubAuthority': [8, ['array', 1, ['unsigned long']]],
    'SubAuthorityCount': [1, ['unsigned char']],
   }],
   '_SID_AND_ATTRIBUTES': [16, {
    'Sid': [0, ['pointer64', ['void']]],
   }],
   '_SID_IDENTIFIER_AUTHORITY': [6, {
    'Value': [0, ['array', 6, ['unsigned char']]],
   }],
   '_SUBSECTION': [56, {
    'ControlArea': [0, ['pointer64', ['_CONTROL_AREA']]],
   }],
   '_TOKEN': [784, {
    'UserAndGroupCount': [120, ['unsigned long']],
    'UserAndGroups': [144, ['pointer64', ['_SID_AND_ATTRIBUTES']]],
   }],
   '_UNICODE_STRING': [16, {
    'Buffer': [8, ['pointer64', ['unsigned short']]],
    'Length': [0, ['unsigned short']],
   }],
   '__unnamed_152f': [8, {
    'Flink': [0, ['unsigned long long']],
   }],
   '__unnamed_1536': [4, {
    'e1': [2, ['_MMPFNENTRY']],
   }],
   '__unnamed_153e': [8, {
    'PteFrame': [0, ['BitField', {
     'end_bit': 52,
     'native_type': 'unsigned long long',
     'start_bit': 0,
    }]],
   }],
   '__unnamed_157b': [4, {
    'Flags': [0, ['_MMSECTION_FLAGS']],
   }],
   '__unnamed_15c2': [8, {
    'VadFlags': [0, ['_MMVAD_FLAGS']],
   }],
   '__unnamed_15d2': [4, {
    'VadFlags2': [0, ['_MMVAD_FLAGS2']],
   }],
  },
 },
 'WinXPSP2x86': {
  'metadata': {
   'Constants': 'Constants.json',
   'ProfileClass': 'WinXPSP2x86',
   'VTypes': 'vtypes.json',
  },
  'vtypes.json': {
   'LIST_ENTRY32': [8, {
    'Blink': [4, ['unsigned long']],
    'Flink': [0, ['unsigned long']],
   }],
   'LIST_ENTRY64': [16, {
   }],
   '_CLIENT_ID': [8, {
    'UniqueProcess': [0, ['pointer', ['void']]],
    'UniqueThread': [4, ['pointer', ['void']]],
   }],
   '_CMHIVE': [1180, {
    'FileFullPath': [584, ['_UNICODE_STRING']],
    'FileUserName': [592, ['_UNICODE_STRING']],
    'Hive': [0, ['_HHIVE']],
    'HiveList': [548, ['_LIST_ENTRY']],
   }],
   '_CM_KEY_BODY': [68, {
    'KeyControlBlock': [4, ['pointer', ['_CM_KEY_CONTROL_BLOCK']]],
   }],
   '_CM_KEY_CONTROL_BLOCK': [72, {
    'NameBlock': [28, ['pointer', ['_CM_NAME_CONTROL_BLOCK']]],
    'ParentKcb': [24, ['pointer', ['_CM_KEY_CONTROL_BLOCK']]],
   }],
   '_CM_KEY_INDEX': [8, {
    'Count': [2, ['unsigned short']],
    'List': [4, ['array', 1, ['unsigned long']]],
    'Signature': [0, ['unsigned short']],
   }],
   '_CM_KEY_NODE': [80, {
    'Flags': [2, ['unsigned short']],
    'LastWriteTime': [4, ['_LARGE_INTEGER']],
    'Signature': [0, ['unsigned short']],
    'SubKeyCounts': [20, ['array', 2, ['unsigned long']]],
    'SubKeyLists': [28, ['array', 2, ['unsigned long']]],
   }],
   '_CM_NAME_CONTROL_BLOCK': [16, {
    'Name': [14, ['array', 1, ['unsigned short']]],
    'NameLength': [12, ['unsigned short']],
   }],
   '_CONTROL_AREA': [48, {
    'DereferenceList': [4, ['_LIST_ENTRY']],
    'FilePointer': [36, ['pointer', ['_FILE_OBJECT']]],
    'NumberOfMappedViews': [20, ['unsigned long']],
    'NumberOfPfnReferences': [16, ['unsigned long']],
    'NumberOfSectionReferences': [12, ['unsigned long']],
    'NumberOfUserReferences': [28, ['unsigned long']],
    'Segment': [0, ['pointer', ['_SEGMENT']]],
    'WaitingForDeletion': [40, ['pointer', ['_EVENT_COUNTER']]],
    'u': [32, ['__unnamed_1224']],
   }],
   '_DBGKD_GET_VERSION64': [40, {
    'KernBase': [16, ['unsigned long long']],
    'MajorVersion': [0, ['unsigned short']],
    'MinorVersion': [2, ['unsigned short']],
    'PsLoadedModuleList': [24, ['unsigned long long']],
   }],
   '_DEVICE_OBJECT': [184, {
    'AttachedDevice': [16, ['pointer', ['_DEVICE_OBJECT']]],
    'DeviceType': [44, ['unsigned long']],
    'DriverObject': [8, ['pointer', ['_DRIVER_OBJECT']]],
    'NextDevice': [12, ['pointer', ['_DEVICE_OBJECT']]],
   }],
   '_DISPATCHER_HEADER': [16, {
    'SignalState': [4, ['long']],
    'Type': [0, ['unsigned char']],
   }],
   '_DRIVER_EXTENSION': [28, {
    'ServiceKeyName': [12, ['_UNICODE_STRING']],
   }],
   '_DRIVER_OBJECT': [168, {
    'DeviceObject': [4, ['pointer', ['_DEVICE_OBJECT']]],
    'DriverName': [28, ['_UNICODE_STRING']],
    'DriverSize': [16, ['unsigned long']],
    'DriverStart': [12, ['pointer', ['void']]],
    'DriverStartIo': [48, ['pointer', ['void']]],
    'MajorFunction': [56, ['array', 28, ['pointer', ['void']]]],
   }],
   '_DUAL': [220, {
    'Length': [0, ['unsigned long']],
    'Map': [4, ['pointer', ['_HMAP_DIRECTORY']]],
   }],
   '_EPROCESS': [608, {
    'ActiveProcessLinks': [136, ['_LIST_ENTRY']],
    'ActiveThreads': [416, ['unsigned long']],
    'AddressCreationLock': [240, ['_FAST_MUTEX']],
    'AddressSpaceInitialized': [584, ['BitField', {
     'end_bit': 12,
     'native_type': 'unsigned long',
     'start_bit': 10,
    }]],
    'AweInfo': [496, ['pointer', ['void']]],
    'BreakOnTermination': [584, ['BitField', {
     'end_bit': 14,
     'native_type': 'unsigned long',
     'start_bit': 13,
    }]],
    'CloneRoot': [292, ['pointer', ['void']]],
    'CommitCharge': [168, ['unsigned long']],
    'CommitChargeLimit': [488, ['unsigned long']],
    'CommitChargePeak': [492, ['unsigned long']],
    'Cookie': [600, ['unsigned long']],
    'CreateReported': [584, ['BitField', {
     'end_bit': 1,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'CreateTime': [112, ['_LARGE_INTEGER']],
    'DebugPort': [188, ['pointer', ['void']]],
    'DefaultHardErrorProcessing': [424, ['unsigned long']],
    'DeviceMap': [348, ['pointer', ['void']]],
    'ExceptionPort': [192, ['pointer', ['void']]],
    'ExitStatus': [588, ['long']],
    'ExitTime': [120, ['_LARGE_INTEGER']],
    'Filler': [360, ['unsigned long long']],
    'Flags': [584, ['unsigned long']],
    'ForkFailed': [584, ['BitField', {
     'end_bit': 9,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
    'ForkInProgress': [276, ['pointer', ['_ETHREAD']]],
    'GrantedAccess': [420, ['unsigned long']],
    'HardwareTrigger': [280, ['unsigned long']],
    'HasAddressSpace': [584, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 18,
    }]],
    'HasPhysicalVad': [584, ['BitField', {
     'end_bit': 10,
     'native_type': 'unsigned long',
     'start_bit': 9,
    }]],
    'HyperSpaceLock': [272, ['unsigned long']],
    'ImageFileName': [372, ['array', 16, ['unsigned char']]],
    'InheritedFromUniqueProcessId': [332, ['pointer', ['void']]],
    'InjectInpageErrors': [584, ['BitField', {
     'end_bit': 21,
     'native_type': 'unsigned long',
     'start_bit': 20,
    }]],
    'Job': [308, ['pointer', ['_EJOB']]],
    'JobLinks': [388, ['_LIST_ENTRY']],
    'JobStatus': [580, ['unsigned long']],
    'LastFaultCount': [568, ['unsigned long']],
    'LastThreadExitStatus': [428, ['long']],
    'LaunchPrefetched': [584, ['BitField', {
     'end_bit': 20,
     'native_type': 'unsigned long',
     'start_bit': 19,
    }]],
    'LdtInformation': [336, ['pointer', ['void']]],
    'LockedPagesList': [396, ['pointer', ['void']]],
    'ModifiedPageCount': [572, ['unsigned long']],
    'NextPageColor': [592, ['unsigned short']],
    'NoDebugInherit': [584, ['BitField', {
     'end_bit': 2,
     'native_type': 'unsigned long',
     'start_bit': 1,
    }]],
    'NumberOfLockedPages': [300, ['unsigned long']],
    'NumberOfPrivatePages': [296, ['unsigned long']],
    'NumberOfVads': [576, ['unsigned long']],
    'ObjectTable': [196, ['pointer', ['_HANDLE_TABLE']]],
    'OtherOperationCount': [456, ['_LARGE_INTEGER']],
    'OtherTransferCount': [480, ['_LARGE_INTEGER']],
    'OutswapEnabled': [584, ['BitField', {
     'end_bit': 7,
     'native_type': 'unsigned long',
     'start_bit': 6,
    }]],
    'Outswapped': [584, ['BitField', {
     'end_bit': 8,
     'native_type': 'unsigned long',
     'start_bit': 7,
    }]],
    'OverrideAddressSpace': [584, ['BitField', {
     'end_bit': 18,
     'native_type': 'unsigned long',
     'start_bit': 17,
    }]],
    'PaeTop': [412, ['pointer', ['void']]],
    'PageDirectoryPte': [360, ['_HARDWARE_PTE']],
    'Pcb': [0, ['_KPROCESS']],
    'PeakVirtualSize': [172, ['unsigned long']],
    'Peb': [432, ['pointer', ['_PEB']]],
    'PhysicalVadList': [352, ['_LIST_ENTRY']],
    'PrefetchTrace': [436, ['_EX_FAST_REF']],
    'PriorityClass': [596, ['unsigned char']],
    'ProcessDelete': [584, ['BitField', {
     'end_bit': 4,
     'native_type': 'unsigned long',
     'start_bit': 3,
    }]],
    'ProcessExiting': [584, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned long',
     'start_bit': 2,
    }]],
    'ProcessInSession': [584, ['BitField', {
     'end_bit': 17,
     'native_type': 'unsigned long',
     'start_bit': 16,
    }]],
    'ProcessLock': [108, ['_EX_PUSH_LOCK']],
    'QuotaBlock': [320, ['pointer', ['_EPROCESS_QUOTA_BLOCK']]],
    'QuotaPeak': [156, ['array', 3, ['unsigned long']]],
    'QuotaUsage': [144, ['array', 3, ['unsigned long']]],
    'ReadOperationCount': [440, ['_LARGE_INTEGER']],
    'ReadTransferCount': [464, ['_LARGE_INTEGER']],
    'RundownProtect': [128, ['_EX_RUNDOWN_REF']],
    'SeAuditProcessCreationInfo': [500, ['_SE_AUDIT_PROCESS_CREATION_INFO']],
    'SectionBaseAddress': [316, ['pointer', ['void']]],
    'SectionObject': [312, ['pointer', ['void']]],
    'SecurityPort': [408, ['pointer', ['void']]],
    'Session': [368, ['pointer', ['void']]],
    'SessionCreationUnderway': [584, ['BitField', {
     'end_bit': 15,
     'native_type': 'unsigned long',
     'start_bit': 14,
    }]],
    'SessionProcessLinks': [180, ['_LIST_ENTRY']],
    'SetTimerResolution': [584, ['BitField', {
     'end_bit': 13,
     'native_type': 'unsigned long',
     'start_bit': 12,
    }]],
    'SubSystemMajorVersion': [595, ['unsigned char']],
    'SubSystemMinorVersion': [594, ['unsigned char']],
    'SubSystemVersion': [594, ['unsigned short']],
    'ThreadListHead': [400, ['_LIST_ENTRY']],
    'Token': [200, ['_EX_FAST_REF']],
    'UniqueProcessId': [132, ['pointer', ['void']]],
    'Unused': [584, ['BitField', {
     'end_bit': 30,
     'native_type': 'unsigned long',
     'start_bit': 25,
    }]],
    'Unused1': [584, ['BitField', {
     'end_bit': 31,
     'native_type': 'unsigned long',
     'start_bit': 30,
    }]],
    'Unused2': [584, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 31,
    }]],
    'Unused3': [584, ['BitField', {
     'end_bit': 23,
     'native_type': 'unsigned long',
     'start_bit': 22,
    }]],
    'Unused4': [584, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 23,
    }]],
    'VadFreeHint': [340, ['pointer', ['void']]],
    'VadHint': [288, ['pointer', ['void']]],
    'VadRoot': [284, ['pointer', ['void']]],
    'VdmAllowed': [584, ['BitField', {
     'end_bit': 25,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
    'VdmObjects': [344, ['pointer', ['void']]],
    'VirtualSize': [176, ['unsigned long']],
    'Vm': [504, ['_MMSUPPORT']],
    'VmDeleted': [584, ['BitField', {
     'end_bit': 6,
     'native_type': 'unsigned long',
     'start_bit': 5,
    }]],
    'VmTopDown': [584, ['BitField', {
     'end_bit': 22,
     'native_type': 'unsigned long',
     'start_bit': 21,
    }]],
    'Win32Process': [304, ['pointer', ['void']]],
    'Win32WindowStation': [328, ['pointer', ['void']]],
    'WorkingSetAcquiredUnsafe': [597, ['unsigned char']],
    'WorkingSetLock': [204, ['_FAST_MUTEX']],
    'WorkingSetPage': [236, ['unsigned long']],
    'WorkingSetWatch': [324, ['pointer', ['_PAGEFAULT_HISTORY']]],
    'Wow64SplitPages': [584, ['BitField', {
     'end_bit': 5,
     'native_type': 'unsigned long',
     'start_bit': 4,
    }]],
    'WriteOperationCount': [448, ['_LARGE_INTEGER']],
    'WriteTransferCount': [472, ['_LARGE_INTEGER']],
    'WriteWatch': [584, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned long',
     'start_bit': 15,
    }]],
   }],
   '_ETHREAD': [600, {
    'Cid': [492, ['_CLIENT_ID']],
    'CreateTime': [448, ['_LARGE_INTEGER']],
    'ExitTime': [456, ['_LARGE_INTEGER']],
    'KeyedWaitSemaphore': [500, ['_KSEMAPHORE']],
    'StartAddress': [548, ['pointer', ['void']]],
    'Tcb': [0, ['_KTHREAD']],
    'ThreadsProcess': [544, ['pointer', ['_EPROCESS']]],
   }],
   '_EX_FAST_REF': [4, {
    'Object': [0, ['pointer', ['void']]],
   }],
   '_EX_PUSH_LOCK': [4, {
   }],
   '_EX_RUNDOWN_REF': [4, {
   }],
   '_FAST_MUTEX': [32, {
   }],
   '_FILE_OBJECT': [112, {
    'DeleteAccess': [40, ['unsigned char']],
    'DeviceObject': [4, ['pointer', ['_DEVICE_OBJECT']]],
    'FileName': [48, ['_UNICODE_STRING']],
    'ReadAccess': [38, ['unsigned char']],
    'SharedDelete': [43, ['unsigned char']],
    'SharedRead': [41, ['unsigned char']],
    'SharedWrite': [42, ['unsigned char']],
    'WriteAccess': [39, ['unsigned char']],
   }],
   '_HANDLE_TABLE': [68, {
    'HandleCount': [60, ['long']],
    'HandleTableList': [28, ['_LIST_ENTRY']],
    'TableCode': [0, ['unsigned long']],
   }],
   '_HANDLE_TABLE_ENTRY': [8, {
    'GrantedAccess': [4, ['unsigned long']],
    'Object': [0, ['pointer', ['void']]],
   }],
   '_HARDWARE_PTE': [4, {
   }],
   '_HHIVE': [528, {
    'BaseBlock': [36, ['pointer', ['_HBASE_BLOCK']]],
    'Flat': [64, ['unsigned char']],
    'Signature': [0, ['unsigned long']],
    'Storage': [88, ['array', 2, ['_DUAL']]],
   }],
   '_HMAP_DIRECTORY': [4096, {
    'Directory': [0, ['array', 1024, ['pointer', ['_HMAP_TABLE']]]],
   }],
   '_HMAP_ENTRY': [16, {
    'BlockAddress': [0, ['unsigned long']],
   }],
   '_HMAP_TABLE': [8192, {
    'Table': [0, ['array', 512, ['_HMAP_ENTRY']]],
   }],
   '_IMAGE_DATA_DIRECTORY': [8, {
    'Size': [4, ['unsigned long']],
    'VirtualAddress': [0, ['unsigned long']],
   }],
   '_IMAGE_DOS_HEADER': [64, {
    'e_lfanew': [60, ['long']],
    'e_magic': [0, ['unsigned short']],
   }],
   '_IMAGE_NT_HEADERS': [248, {
    'OptionalHeader': [24, ['_IMAGE_OPTIONAL_HEADER']],
    'Signature': [0, ['unsigned long']],
   }],
   '_IMAGE_OPTIONAL_HEADER': [224, {
    'DataDirectory': [96, ['array', 16, ['_IMAGE_DATA_DIRECTORY']]],
    'Magic': [0, ['unsigned short']],
    'MajorOperatingSystemVersion': [40, ['unsigned short']],
    'MinorOperatingSystemVersion': [42, ['unsigned short']],
    'SizeOfImage': [56, ['unsigned long']],
   }],
   '_KAPC_STATE': [24, {
    'Process': [16, ['pointer', ['_KPROCESS']]],
   }],
   '_KDPC': [32, {
    'DeferredRoutine': [12, ['pointer', ['void']]],
   }],
   '_KMUTANT': [32, {
    'Header': [0, ['_DISPATCHER_HEADER']],
    'OwnerThread': [24, ['pointer', ['_KTHREAD']]],
   }],
   '_KPCR': [3440, {
    'Prcb': [32, ['pointer', ['_KPRCB']]],
    'PrcbData': [288, ['_KPRCB']],
   }],
   '_KPRCB': [3152, {
    'Number': [16, ['unsigned char']],
   }],
   '_KPROCESS': [108, {
    'DirectoryTableBase': [24, ['array', 2, ['unsigned long']]],
   }],
   '_KSEMAPHORE': [20, {
    'Header': [0, ['_DISPATCHER_HEADER']],
   }],
   '_KTHREAD': [448, {
    'ApcState': [52, ['_KAPC_STATE']],
    'SuspendSemaphore': [412, ['_KSEMAPHORE']],
   }],
   '_KTIMER': [40, {
    'Dpc': [32, ['pointer', ['_KDPC']]],
    'DueTime': [16, ['_ULARGE_INTEGER']],
    'Header': [0, ['_DISPATCHER_HEADER']],
    'Period': [36, ['long']],
    'TimerListEntry': [24, ['_LIST_ENTRY']],
   }],
   '_LARGE_INTEGER': [8, {
   }],
   '_LDR_DATA_TABLE_ENTRY': [80, {
    'BaseDllName': [44, ['_UNICODE_STRING']],
    'DllBase': [24, ['pointer', ['void']]],
    'FullDllName': [36, ['_UNICODE_STRING']],
    'InInitializationOrderLinks': [16, ['_LIST_ENTRY']],
    'InLoadOrderLinks': [0, ['_LIST_ENTRY']],
    'InMemoryOrderLinks': [8, ['_LIST_ENTRY']],
    'SizeOfImage': [32, ['unsigned long']],
   }],
   '_LIST_ENTRY': [8, {
    'Blink': [4, ['pointer', ['_LIST_ENTRY']]],
    'Flink': [0, ['pointer', ['_LIST_ENTRY']]],
   }],
   '_MMPFN': [24, {
    'PteAddress': [4, ['pointer', ['_MMPTE']]],
    'u1': [0, ['__unnamed_179d']],
    'u3': [12, ['__unnamed_17a4']],
    'u4': [20, ['__unnamed_17aa']],
   }],
   '_MMPFNENTRY': [4, {
    'PageLocation': [0, ['BitField', {
     'end_bit': 11,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
   }],
   '_MMSECTION_FLAGS': [4, {
    'Accessed': [0, ['BitField', {
     'end_bit': 28,
     'native_type': 'unsigned long',
     'start_bit': 27,
    }]],
    'Based': [0, ['BitField', {
     'end_bit': 7,
     'native_type': 'unsigned long',
     'start_bit': 6,
    }]],
    'BeingCreated': [0, ['BitField', {
     'end_bit': 2,
     'native_type': 'unsigned long',
     'start_bit': 1,
    }]],
    'BeingDeleted': [0, ['BitField', {
     'end_bit': 1,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'BeingPurged': [0, ['BitField', {
     'end_bit': 3,
     'native_type': 'unsigned long',
     'start_bit': 2,
    }]],
    'CollidedFlush': [0, ['BitField', {
     'end_bit': 23,
     'native_type': 'unsigned long',
     'start_bit': 22,
    }]],
    'Commit': [0, ['BitField', {
     'end_bit': 14,
     'native_type': 'unsigned long',
     'start_bit': 13,
    }]],
    'CopyOnWrite': [0, ['BitField', {
     'end_bit': 12,
     'native_type': 'unsigned long',
     'start_bit': 11,
    }]],
    'DebugSymbolsLoaded': [0, ['BitField', {
     'end_bit': 21,
     'native_type': 'unsigned long',
     'start_bit': 20,
    }]],
    'DeleteOnClose': [0, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 18,
    }]],
    'FailAllIo': [0, ['BitField', {
     'end_bit': 5,
     'native_type': 'unsigned long',
     'start_bit': 4,
    }]],
    'File': [0, ['BitField', {
     'end_bit': 8,
     'native_type': 'unsigned long',
     'start_bit': 7,
    }]],
    'FilePointerNull': [0, ['BitField', {
     'end_bit': 20,
     'native_type': 'unsigned long',
     'start_bit': 19,
    }]],
    'FloppyMedia': [0, ['BitField', {
     'end_bit': 15,
     'native_type': 'unsigned long',
     'start_bit': 14,
    }]],
    'GlobalMemory': [0, ['BitField', {
     'end_bit': 18,
     'native_type': 'unsigned long',
     'start_bit': 17,
    }]],
    'GlobalOnlyPerSession': [0, ['BitField', {
     'end_bit': 29,
     'native_type': 'unsigned long',
     'start_bit': 28,
    }]],
    'HadUserReference': [0, ['BitField', {
     'end_bit': 25,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
    'Image': [0, ['BitField', {
     'end_bit': 6,
     'native_type': 'unsigned long',
     'start_bit': 5,
    }]],
    'ImageMappedInSystemSpace': [0, ['BitField', {
     'end_bit': 26,
     'native_type': 'unsigned long',
     'start_bit': 25,
    }]],
    'Networked': [0, ['BitField', {
     'end_bit': 9,
     'native_type': 'unsigned long',
     'start_bit': 8,
    }]],
    'NoCache': [0, ['BitField', {
     'end_bit': 10,
     'native_type': 'unsigned long',
     'start_bit': 9,
    }]],
    'NoChange': [0, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 23,
    }]],
    'NoModifiedWriting': [0, ['BitField', {
     'end_bit': 4,
     'native_type': 'unsigned long',
     'start_bit': 3,
    }]],
    'PhysicalMemory': [0, ['BitField', {
     'end_bit': 11,
     'native_type': 'unsigned long',
     'start_bit': 10,
    }]],
    'Reserve': [0, ['BitField', {
     'end_bit': 13,
     'native_type': 'unsigned long',
     'start_bit': 12,
    }]],
    'Rom': [0, ['BitField', {
     'end_bit': 30,
     'native_type': 'unsigned long',
     'start_bit': 29,
    }]],
    'SetMappedFileIoComplete': [0, ['BitField', {
     'end_bit': 22,
     'native_type': 'unsigned long',
     'start_bit': 21,
    }]],
    'UserReference': [0, ['BitField', {
     'end_bit': 17,
     'native_type': 'unsigned long',
     'start_bit': 16,
    }]],
    'UserWritable': [0, ['BitField', {
     'end_bit': 27,
     'native_type': 'unsigned long',
     'start_bit': 26,
    }]],
    'WasPurged': [0, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned long',
     'start_bit': 15,
    }]],
    'filler': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 30,
    }]],
   }],
   '_MMSUPPORT': [64, {
   }],
   '_MMVAD': [40, {
    'ControlArea': [24, ['pointer', ['_CONTROL_AREA']]],
    'EndingVpn': [4, ['unsigned long']],
    'FirstPrototypePte': [28, ['pointer', ['_MMPTE']]],
    'LastContiguousPte': [32, ['pointer', ['_MMPTE']]],
    'LeftChild': [12, ['pointer', ['_MMVAD']]],
    'RightChild': [16, ['pointer', ['_MMVAD']]],
    'StartingVpn': [0, ['unsigned long']],
    'u': [20, ['__unnamed_144c']],
    'u2': [36, ['__unnamed_144f']],
   }],
   '_MMVAD_FLAGS': [4, {
    'CommitCharge': [0, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'ImageMap': [0, ['BitField', {
     'end_bit': 21,
     'native_type': 'unsigned long',
     'start_bit': 20,
    }]],
    'LargePages': [0, ['BitField', {
     'end_bit': 30,
     'native_type': 'unsigned long',
     'start_bit': 29,
    }]],
    'MemCommit': [0, ['BitField', {
     'end_bit': 31,
     'native_type': 'unsigned long',
     'start_bit': 30,
    }]],
    'NoChange': [0, ['BitField', {
     'end_bit': 23,
     'native_type': 'unsigned long',
     'start_bit': 22,
    }]],
    'PhysicalMapping': [0, ['BitField', {
     'end_bit': 20,
     'native_type': 'unsigned long',
     'start_bit': 19,
    }]],
    'PrivateMemory': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 31,
    }]],
    'Protection': [0, ['BitField', {
     'end_bit': 29,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
    'UserPhysicalPages': [0, ['BitField', {
     'end_bit': 22,
     'native_type': 'unsigned long',
     'start_bit': 21,
    }]],
    'WriteWatch': [0, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 23,
    }]],
   }],
   '_MMVAD_FLAGS2': [4, {
    'CopyOnWrite': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 31,
    }]],
    'ExtendableFile': [0, ['BitField', {
     'end_bit': 30,
     'native_type': 'unsigned long',
     'start_bit': 29,
    }]],
    'FileOffset': [0, ['BitField', {
     'end_bit': 24,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'Inherit': [0, ['BitField', {
     'end_bit': 31,
     'native_type': 'unsigned long',
     'start_bit': 30,
    }]],
    'LongVad': [0, ['BitField', {
     'end_bit': 29,
     'native_type': 'unsigned long',
     'start_bit': 28,
    }]],
    'MultipleSecured': [0, ['BitField', {
     'end_bit': 27,
     'native_type': 'unsigned long',
     'start_bit': 26,
    }]],
    'OneSecured': [0, ['BitField', {
     'end_bit': 26,
     'native_type': 'unsigned long',
     'start_bit': 25,
    }]],
    'ReadOnly': [0, ['BitField', {
     'end_bit': 28,
     'native_type': 'unsigned long',
     'start_bit': 27,
    }]],
    'SecNoChange': [0, ['BitField', {
     'end_bit': 25,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
   }],
   '_MMVAD_LONG': [52, {
    'ControlArea': [24, ['pointer', ['_CONTROL_AREA']]],
    'EndingVpn': [4, ['unsigned long']],
    'FirstPrototypePte': [28, ['pointer', ['_MMPTE']]],
    'LastContiguousPte': [32, ['pointer', ['_MMPTE']]],
    'LeftChild': [12, ['pointer', ['_MMVAD']]],
    'RightChild': [16, ['pointer', ['_MMVAD']]],
    'StartingVpn': [0, ['unsigned long']],
    'u': [20, ['__unnamed_144c']],
    'u2': [36, ['__unnamed_144f']],
   }],
   '_MMVAD_SHORT': [24, {
    'EndingVpn': [4, ['unsigned long']],
    'LeftChild': [12, ['pointer', ['_MMVAD']]],
    'RightChild': [16, ['pointer', ['_MMVAD']]],
    'StartingVpn': [0, ['unsigned long']],
    'u': [20, ['__unnamed_144c']],
   }],
   '_MM_SESSION_SPACE': [4728, {
    'ImageList': [156, ['_LIST_ENTRY']],
    'PagedPoolEnd': [104, ['pointer', ['void']]],
    'PagedPoolStart': [100, ['pointer', ['void']]],
    'ProcessList': [20, ['_LIST_ENTRY']],
    'SessionId': [8, ['unsigned long']],
   }],
   '_OBJECT_HEADER': [32, {
    'Body': [24, ['_QUAD']],
    'HandleCount': [4, ['long']],
    'NameInfoOffset': [12, ['unsigned char']],
    'PointerCount': [0, ['long']],
    'Type': [8, ['pointer', ['_OBJECT_TYPE']]],
   }],
   '_OBJECT_HEADER_NAME_INFO': [16, {
    'Name': [4, ['_UNICODE_STRING']],
   }],
   '_OBJECT_SYMBOLIC_LINK': [32, {
    'CreationTime': [0, ['_LARGE_INTEGER']],
    'LinkTarget': [8, ['_UNICODE_STRING']],
   }],
   '_OBJECT_TYPE': [400, {
    'Name': [64, ['_UNICODE_STRING']],
   }],
   '_PEB': [528, {
    'CSDVersion': [496, ['_UNICODE_STRING']],
    'ImageBaseAddress': [8, ['pointer', ['void']]],
    'Ldr': [12, ['pointer', ['_PEB_LDR_DATA']]],
    'ProcessParameters': [16, ['pointer', ['_RTL_USER_PROCESS_PARAMETERS']]],
   }],
   '_PEB_LDR_DATA': [40, {
    'InInitializationOrderModuleList': [28, ['_LIST_ENTRY']],
    'InLoadOrderModuleList': [12, ['_LIST_ENTRY']],
    'InMemoryOrderModuleList': [20, ['_LIST_ENTRY']],
   }],
   '_POOL_HEADER': [8, {
    'BlockSize': [2, ['BitField', {
     'end_bit': 9,
     'native_type': 'unsigned short',
     'start_bit': 0,
    }]],
    'PoolIndex': [0, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned short',
     'start_bit': 9,
    }]],
    'PoolTag': [4, ['unsigned long']],
    'PoolType': [2, ['BitField', {
     'end_bit': 16,
     'native_type': 'unsigned short',
     'start_bit': 9,
    }]],
   }],
   '_QUAD': [8, {
   }],
   '_RTL_ATOM_TABLE': [68, {
    'Buckets': [64, ['array', 1, ['pointer', ['_RTL_ATOM_TABLE_ENTRY']]]],
    'Signature': [0, ['unsigned long']],
   }],
   '_RTL_ATOM_TABLE_ENTRY': [16, {
    'Atom': [6, ['unsigned short']],
    'Flags': [10, ['unsigned char']],
    'HandleIndex': [4, ['unsigned short']],
    'HashLink': [0, ['pointer', ['_RTL_ATOM_TABLE_ENTRY']]],
    'Name': [12, ['array', 1, ['unsigned short']]],
    'NameLength': [11, ['unsigned char']],
    'ReferenceCount': [8, ['unsigned short']],
   }],
   '_RTL_USER_PROCESS_PARAMETERS': [656, {
    'CommandLine': [64, ['_UNICODE_STRING']],
    'Environment': [72, ['pointer', ['void']]],
   }],
   '_SE_AUDIT_PROCESS_CREATION_INFO': [4, {
   }],
   '_SID': [12, {
    'IdentifierAuthority': [2, ['_SID_IDENTIFIER_AUTHORITY']],
    'Revision': [0, ['unsigned char']],
    'SubAuthority': [8, ['array', 1, ['unsigned long']]],
    'SubAuthorityCount': [1, ['unsigned char']],
   }],
   '_SID_AND_ATTRIBUTES': [8, {
    'Sid': [0, ['pointer', ['void']]],
   }],
   '_SID_IDENTIFIER_AUTHORITY': [6, {
    'Value': [0, ['array', 6, ['unsigned char']]],
   }],
   '_TOKEN': [168, {
    'UserAndGroupCount': [76, ['unsigned long']],
    'UserAndGroups': [104, ['pointer', ['_SID_AND_ATTRIBUTES']]],
   }],
   '_ULARGE_INTEGER': [8, {
    'QuadPart': [0, ['unsigned long long']],
   }],
   '_UNICODE_STRING': [8, {
    'Buffer': [4, ['pointer', ['unsigned short']]],
    'Length': [0, ['unsigned short']],
   }],
   '__unnamed_1224': [4, {
    'Flags': [0, ['_MMSECTION_FLAGS']],
   }],
   '__unnamed_144c': [4, {
    'VadFlags': [0, ['_MMVAD_FLAGS']],
   }],
   '__unnamed_144f': [4, {
    'VadFlags2': [0, ['_MMVAD_FLAGS2']],
   }],
   '__unnamed_179d': [4, {
    'Flink': [0, ['unsigned long']],
   }],
   '__unnamed_17a4': [4, {
    'e1': [0, ['_MMPFNENTRY']],
   }],
   '__unnamed_17aa': [4, {
    'PteFrame': [0, ['BitField', {
     'end_bit': 26,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
   }],
  },
 },
 'WinXPSP3x86': {
  'metadata': {
   'Constants': 'Constants.json',
   'ProfileClass': 'WinXPSP3x86',
   'VTypes': 'vtypes.json',
  },
  'vtypes.json': {
   'LIST_ENTRY32': [8, {
    'Blink': [4, ['unsigned long']],
    'Flink': [0, ['unsigned long']],
   }],
   'LIST_ENTRY64': [16, {
   }],
   '_CONTROL_AREA': [48, {
    'FilePointer': [36, ['pointer', ['_FILE_OBJECT']]],
   }],
   '_EPROCESS': [608, {
    'ActiveProcessLinks': [136, ['_LIST_ENTRY']],
    'ActiveThreads': [416, ['unsigned long']],
    'CreateTime': [112, ['_LARGE_INTEGER']],
    'ExitTime': [120, ['_LARGE_INTEGER']],
    'ImageFileName': [372, ['array', 16, ['unsigned char']]],
    'InheritedFromUniqueProcessId': [332, ['pointer', ['void']]],
    'ObjectTable': [196, ['pointer', ['_HANDLE_TABLE']]],
    'Pcb': [0, ['_KPROCESS']],
    'Session': [368, ['pointer', ['void']]],
    'ThreadListHead': [400, ['_LIST_ENTRY']],
    'UniqueProcessId': [132, ['pointer', ['void']]],
    'VadRoot': [284, ['pointer', ['void']]],
   }],
   '_FILE_OBJECT': [112, {
    'FileName': [48, ['_UNICODE_STRING']],
   }],
   '_HANDLE_TABLE': [68, {
    'HandleCount': [60, ['long']],
   }],
   '_KPROCESS': [108, {
    'DirectoryTableBase': [24, ['array', 2, ['unsigned long']]],
   }],
   '_LIST_ENTRY': [8, {
    'Blink': [4, ['pointer', ['_LIST_ENTRY']]],
    'Flink': [0, ['pointer', ['_LIST_ENTRY']]],
   }],
   '_MMVAD': [40, {
    'ControlArea': [24, ['pointer', ['_CONTROL_AREA']]],
    'EndingVpn': [4, ['unsigned long']],
    'LeftChild': [12, ['pointer', ['_MMVAD']]],
    'RightChild': [16, ['pointer', ['_MMVAD']]],
    'StartingVpn': [0, ['unsigned long']],
    'u': [20, ['__unnamed_1492']],
   }],
   '_MMVAD_FLAGS': [4, {
    'CommitCharge': [0, ['BitField', {
     'end_bit': 19,
     'native_type': 'unsigned long',
     'start_bit': 0,
    }]],
    'PrivateMemory': [0, ['BitField', {
     'end_bit': 32,
     'native_type': 'unsigned long',
     'start_bit': 31,
    }]],
    'Protection': [0, ['BitField', {
     'end_bit': 29,
     'native_type': 'unsigned long',
     'start_bit': 24,
    }]],
   }],
   '_MMVAD_LONG': [52, {
    'ControlArea': [24, ['pointer', ['_CONTROL_AREA']]],
    'EndingVpn': [4, ['unsigned long']],
    'StartingVpn': [0, ['unsigned long']],
    'u': [20, ['__unnamed_1492']],
   }],
   '_MMVAD_SHORT': [24, {
    'EndingVpn': [4, ['unsigned long']],
    'StartingVpn': [0, ['unsigned long']],
    'u': [20, ['__unnamed_1492']],
   }],
   '_MM_SESSION_SPACE': [4728, {
    'SessionId': [8, ['unsigned long']],
   }],
   '_UNICODE_STRING': [8, {
    'Buffer': [4, ['pointer', ['unsigned short']]],
    'Length': [0, ['unsigned short']],
   }],
   '__unnamed_1492': [4, {
    'VadFlags': [0, ['_MMVAD_FLAGS']],
   }],
  },
 },
}

